@@ -51,27 +51,23 @@ use 5.008008;
 use strict;
 use warnings;
 use vars qw($VERSION %VALID_TYPES);
-use subs qw(
-    new name code signature help version hidden add_signature
-    delete_signature make_sig_table match_signature reload load_xpl_file
-);
 
 use File::Spec;
 use Scalar::Util 'blessed';
 
 use RPC::XML 'smart_encode';
 
-# This module also provides RPC::XML::Method
+# This module also provides RPC::XML::Method and RPC::XML::Function
 ## no critic (ProhibitMultiplePackages)
 
-$VERSION = '1.29';
+$VERSION = '1.31';
 $VERSION = eval $VERSION;    ## no critic (ProhibitStringyEval)
 
 # This should match the set of type-classes defined in RPC::XML.pm. Note that
-# we use "datetime_iso8601" instead of "dateTime.iso8601", because that is how
+# we use "dateTime.iso8601" instead of "datetime_iso8601", because that is how
 # it has to be in the signature.
 %VALID_TYPES = map { $_ => 1 }
-    (qw(int i4 i8 double string boolean datetime_iso8601 nil array struct
+    (qw(int i4 i8 double string boolean dateTime.iso8601 nil array struct
         base64));
 
 ###############################################################################
@@ -224,7 +220,7 @@ sub make_sig_table
             return "$me: Unknown return type '$return'";
         }
         # Not going to add List::MoreUtils to my dependencies list, so suppress
-        # this ciritic flag:
+        # this critic flag:
         ## no critic (ProhibitBooleanGrep)
         if (grep { ! $VALID_TYPES{$_} } @rest)
         {
@@ -666,6 +662,10 @@ sub call
             "[$signature]"
         );
     }
+    elsif ($resptype eq 'dateTime.iso8601')
+    {
+        $resptype = 'datetime_iso8601';
+    }
 
     # Set these in case the server object is part of the param list
     local $srv->{signature} =          ## no critic (ProhibitLocalVars)
@@ -868,7 +868,7 @@ arguments to the constructor are variable in nature, depending on the type:
 
 If there is exactly on argument that is not a reference, it is assumed to be a
 filename from which the method is to be loaded. This is presumed to be in the
-B<XPL> format descibed below (see L</"XPL File Structure">). If the file
+B<XPL> format described below (see L</"XPL File Structure">). If the file
 cannot be opened, or if once opened cannot be parsed, an error is raised.
 
 =item HASHREF
@@ -882,7 +882,7 @@ when methods were implemented simply as hash references.
 
 If there is more than one argument in the list, then the list is assumed to be
 a sort of "ersatz" hash construct, in that one of the keys (C<signature>) is
-allowed to "stack" if it occur multiple times. Otherwise, any keys that occur
+allowed to "stack" if it occurs multiple times. Otherwise, any keys that occur
 multiple times overwrite the previous value:
 
 =over 12
@@ -937,7 +937,7 @@ clone will not impact the original.
 
 Returns the name by which the server is advertising the method. Unlike the
 next few accessors, this cannot be changed on an object. In order to
-streamline the managment of methods within the server classes, this must
+streamline the management of methods within the server classes, this must
 persist. However, the other elements may be used in the creation of a new
 object, which may then be added to the server, if the name absolutely must
 change.
@@ -1087,20 +1087,20 @@ other packages than this one, or useful in other contexts than this one.
 
 The lightweight DTD for the layout can be summarized as:
 
-        <!ELEMENT  proceduredef  (name, namespace?, version?, hidden?,
-                                  signature+, help?, code)>
-        <!ELEMENT  methoddef     (name, namespace?, version?, hidden?,
-                                  signature+, help?, code)>
-        <!ELEMENT  functiondef   (name, namespace?, version?, hidden?,
-                                  signature+, help?, code)>
-        <!ELEMENT  name       (#PCDATA)>
-        <!ELEMENT  namespace  (#PCDATA)>
-        <!ELEMENT  version    (#PCDATA)>
-        <!ELEMENT  hidden     EMPTY>
-        <!ELEMENT  signature  (#PCDATA)>
-        <!ELEMENT  help       (#PCDATA)>
-        <!ELEMENT  code       (#PCDATA)>
-        <!ATTLIST  code       language (#PCDATA)>
+    <!ELEMENT  proceduredef  (name, namespace?, version?, hidden?,
+                              signature+, help?, code)>
+    <!ELEMENT  methoddef     (name, namespace?, version?, hidden?,
+                              signature+, help?, code)>
+    <!ELEMENT  functiondef   (name, namespace?, version?, hidden?,
+                              signature+, help?, code)>
+    <!ELEMENT  name       (#PCDATA)>
+    <!ELEMENT  namespace  (#PCDATA)>
+    <!ELEMENT  version    (#PCDATA)>
+    <!ELEMENT  hidden     EMPTY>
+    <!ELEMENT  signature  (#PCDATA)>
+    <!ELEMENT  help       (#PCDATA)>
+    <!ELEMENT  code       (#PCDATA)>
+    <!ATTLIST  code       language (#PCDATA)>
 
 The containing tag is always one of C<< <methoddef> >>, C<< <proceduredef> >>
 or C<< <functiondef> >>. The tags that specify name, signatures and the code
