@@ -1,6 +1,6 @@
 ###############################################################################
 #
-# This file copyright (c) 2001-2011 Randy J. Ray, all rights reserved
+# This file copyright (c) 2001-2014 Randy J. Ray, all rights reserved
 #
 # Copying and distribution are permitted under the terms of the Artistic
 # License 2.0 (http://www.opensource.org/licenses/artistic-license-2.0.php) or
@@ -47,14 +47,15 @@
 #                   timeout
 #                   server_fault
 #
-#   Libraries:      AutoLoader
-#                   HTTP::Daemon (conditionally)
+#   Libraries:      HTTP::Daemon (conditionally)
 #                   HTTP::Response
 #                   HTTP::Status
 #                   URI
+#                   Scalar::Util
 #                   RPC::XML
 #                   RPC::XML::ParserFactory
 #                   RPC::XML::Procedure
+#                   Compress::Raw::Zlib is used if available
 #
 #   Global Consts:  $VERSION
 #                   $INSTALL_DIR
@@ -67,17 +68,19 @@ package RPC::XML::Server;
 use 5.008008;
 use strict;
 use warnings;
-use vars qw($VERSION $INSTALL_DIR %FAULT_TABLE  @XPL_PATH
+use vars qw($VERSION $INSTALL_DIR %FAULT_TABLE  @XPL_PATH %CLASS_MAP
             $IO_SOCKET_SSL_HACK_NEEDED $COMPRESSION_AVAILABLE);
 
 use Carp qw(carp croak);
 use File::Spec;
+use File::Temp;
 use IO::Handle;
+use Module::Load;
+use Scalar::Util 'blessed';
 
 use HTTP::Status;
 use HTTP::Response;
 use URI;
-use Scalar::Util 'blessed';
 
 use RPC::XML;
 use RPC::XML::ParserFactory;
@@ -96,10 +99,8 @@ BEGIN
     $IO_SOCKET_SSL_HACK_NEEDED = 1;
 
     # Check for compression support
-    if (! eval { require Compress::Zlib; $COMPRESSION_AVAILABLE = 'deflate'; })
-    {
-        $COMPRESSION_AVAILABLE = q{};
-    }
+    $COMPRESSION_AVAILABLE =
+        (eval { load Compress::Zlib; 1; }) ? 'deflate' : q{};
 
     # Set up the initial table of fault-types and their codes/messages
     %FAULT_TABLE = (
@@ -108,10 +109,16 @@ BEGIN
         badsignature => [ 201 => 'Method signature error: %s' ],
         execerror    => [ 300 => 'Code execution error: %s' ],
     );
+
+    # This is used by add_method to map "types" to instantiation classes
+    %CLASS_MAP = (
+        method    => 'RPC::XML::Method',
+        procedure => 'RPC::XML::Procedure',
+        function  => 'RPC::XML::Function',
+    );
 }
 
-$VERSION = '1.68';
-$VERSION = eval $VERSION; ## no critic (ProhibitStringyEval)
+$VERSION = '1.75';
 
 ###############################################################################
 #
@@ -138,7 +145,12 @@ sub new ## no critic (ProhibitExcessComplexity)
         $srv_name
     );
 
-    $class = ref($class) || $class;
+    # Don't accept a blessed value for $class
+    if (ref $class)
+    {
+        return __PACKAGE__ . '::new: Must be called as a static method';
+    }
+
     $self = bless {}, $class;
 
     $srv_version = delete $args{server_version} || $self->version;
@@ -158,7 +170,7 @@ sub new ## no critic (ProhibitExcessComplexity)
         $port  = delete $args{port}  || q{};
         $queue = delete $args{queue} || 5;
         $http  = HTTP::Daemon->new(
-            Reuse => 1,
+            ReuseAddr => 1,
             ($host  ? (LocalHost => $host)  : ()),
             ($port  ? (LocalPort => $port)  : ()),
             ($queue ? (Listen    => $queue) : ())
@@ -204,6 +216,9 @@ sub new ## no critic (ProhibitExcessComplexity)
         $args{parser} ? @{delete $args{parser}} : ()
     );
 
+    # Add the basic paths (content of @XPL_PATH) to our local XPL path
+    push @{$self->{__xpl_path}}, @XPL_PATH;
+
     # Set up the default methods unless requested not to
     if (! delete $args{no_default})
     {
@@ -233,7 +248,7 @@ sub new ## no critic (ProhibitExcessComplexity)
 
     # Parameters to control the point at which messages are shunted to temp
     # files due to size, and where to home the temp files. Start with a size
-    # threshhold of 1Meg and no specific dir (which will fall-through to the
+    # threshold of 1Meg and no specific dir (which will fall-through to the
     # tmpdir() method of File::Spec).
     $self->{__message_file_thresh} = delete $args{message_file_thresh} ||
         1_048_576;
@@ -392,6 +407,8 @@ sub xpl_path
 #                   $self     in      ref       Object to add to
 #                   $meth     in      scalar    Hash ref of data or file name
 #
+#   Globals:        %CLASS_MAP
+#
 #   Returns:        Success:    $self
 #                   Failure:    error string
 #
@@ -416,11 +433,22 @@ sub add_method
     }
     elsif (ref $meth eq 'HASH')
     {
+        # Make a copy of the contents of $meth, so we don't make permanent
+        # changes:
+        my %meth_copy = map { $_ => $meth->{$_} } (keys %{$meth});
+
         # If the type of this method is not set, default to "method". The
         # add_procedure and add_function calls should set this as needed.
-        $meth->{type} ||= 'method';
-        my $class = 'RPC::XML::' . ucfirst $meth->{type};
-        $meth = $class->new($meth);
+        my $type = delete $meth_copy{type} || 'method';
+
+        if (! (my $class = $CLASS_MAP{lc $type}))
+        {
+            return "$me: Unknown type: $type";
+        }
+        else
+        {
+            $meth = $class->new(\%meth_copy);
+        }
     }
     elsif (! (blessed $meth and $meth->isa('RPC::XML::Procedure')))
     {
@@ -463,7 +491,7 @@ sub add_procedure
 
 ###############################################################################
 #
-#   Sub Name:       add_procedure
+#   Sub Name:       add_function
 #
 #   Description:    This filters through to add_method, but if the passed-in
 #                   value is a hash reference forces the "type" to be
@@ -512,10 +540,10 @@ sub method_from_file
     if (! File::Spec->file_name_is_absolute($file))
     {
         my $path;
-        for my $dir (@{$self->xpl_path}, @XPL_PATH)
+        for my $dir (@{$self->xpl_path})
         {
             $path = File::Spec->catfile($dir, $file);
-            if (-e $path)
+            if (-f $path)
             {
                 $file = File::Spec->canonpath($path);
                 last;
@@ -661,7 +689,7 @@ sub server_loop ## no critic (RequireArgUnpacking,ProhibitExcessComplexity)
             };
             if ((! $eval_return) && $@)
             {
-                warn "Cannot close conection: $@\n";
+                warn "Cannot close connection: $@\n";
             }
 
             undef $conn;    # Free up any lingering resources
@@ -792,15 +820,20 @@ sub process_request ## no critic (ProhibitExcessComplexity)
     my $conn = shift;
 
     my (
-        $req,     $reqxml,     $resp,     $respxml,  $do_compress,
-        $parser,  $com_engine, $length,   $read,     $buf,
-        $resp_fh, $tmpdir,     $peeraddr, $peerhost, $peerport
+        $req,     $reqxml,     $resp,       $respxml,  $do_compress,
+        $parser,  $com_engine, $length,     $read,     $buf,
+        $resp_fh, $tmpdir,     $peerfamily, $peeraddr, $peerhost, $peerport,
     );
 
     my $me = ref($self) . '::process_request';
-    if (! ($conn && ref $conn))
+    if (! $conn)
     {
+        # Maintain compatibility with Net::Server 0.99, which does not pass
+        # the connection object at all:
         $conn = $self->{server}->{client};
+    }
+    if (ref($conn) =~ /^Net::Server::Proto/)
+    {
         bless $conn, 'HTTP::Daemon::ClientConn';
         ${*{$conn}}{'httpd_daemon'} = $self;
 
@@ -822,6 +855,7 @@ sub process_request ## no critic (ProhibitExcessComplexity)
 
     # These will be attached to any and all request objects that are
     # (successfully) read from $conn.
+    $peerfamily = $conn->sockdomain;
     $peeraddr = $conn->peeraddr;
     $peerport = $conn->peerport;
     $peerhost = $conn->peerhost;
@@ -840,6 +874,7 @@ sub process_request ## no critic (ProhibitExcessComplexity)
             # Get a XML::Parser::ExpatNB object
             $parser = $self->parser->parse();
 
+            $do_compress = 0; # in case it was set for a previous response
             if (($req->content_encoding || q{}) =~ $self->compress_re)
             {
                 if (! $self->compress)
@@ -951,6 +986,7 @@ sub process_request ## no critic (ProhibitExcessComplexity)
             {
                 # Set localized keys on $self, based on the connection info
                 ## no critic (ProhibitLocalVars)
+                local $self->{peerfamily} = $peerfamily;
                 local $self->{peeraddr} = $peeraddr;
                 local $self->{peerhost} = $peerhost;
                 local $self->{peerport} = $peerport;
@@ -976,7 +1012,7 @@ sub process_request ## no critic (ProhibitExcessComplexity)
                 $resp->header(Content_Encoding => $self->compress);
             }
             # Next step, determine the response disposition. If it is above the
-            # threshhold for a requested file cut-off, send it to a temp file
+            # threshold for a requested file cut-off, send it to a temp file
             if ($self->message_file_thresh &&
                 $self->message_file_thresh < $respxml->length)
             {
@@ -1080,7 +1116,7 @@ sub process_request ## no critic (ProhibitExcessComplexity)
             else
             {
                 # Treat the content strictly in-memory
-                utf8::downgrade($buf = $respxml->as_string);
+                utf8::encode($buf = $respxml->as_string);
                 if ($do_compress)
                 {
                     $buf = Compress::Zlib::compress($buf);
@@ -1811,7 +1847,7 @@ would be larger than this when stringified is instead written to an anonynous
 temporary file, and spooled from there instead. This is useful for cases in
 which the request includes B<RPC::XML::base64> objects that are themselves
 spooled from file-handles. This test is independent of compression, so even
-if compression of a request would drop it below this threshhold, it will be
+if compression of a request would drop it below this threshold, it will be
 spooled anyway. The file itself is created via File::Temp with C<UNLINK> set,
 so once it is freed the disk space is immediately freed.
 
@@ -2130,7 +2166,7 @@ The calling server object shares the methods/procedures/functions listed in
 B<NAMES> with the source-server passed as the first object. The source must
 derive from this package in order for this operation to be permitted. At least
 one method must be specified, and all are specified by name (not by object
-refernce). Both objects will reference the same exact B<RPC::XML::Procedure>
+reference). Both objects will reference the same exact B<RPC::XML::Procedure>
 (or derivative thereof) object in this case, meaning that call-statistics and
 the like will reflect the combined data. If one or more of the passed names are
 not present on the source server, an error message is returned and none are
@@ -2258,14 +2294,22 @@ reference containing one or more datatypes, each a simple string. The first
 of the datatypes specifies the expected return type. The remainder (if any)
 refer to the arguments themselves.
 
+=item peerfamily
+
+This is the address family, C<AF_INET> or C<AF_INET6>, of a network address of
+the client that has connected and made the current request. It is required
+for unpacking C<peeraddr> properly.
+
 =item peeraddr
 
-This is the address part of a packed B<SOCKADDR_IN> structure, as returned by
-L<Socket/pack_sockaddr_in>, which contains the address of the client that has
+This is the address part of a packed B<SOCKADDR_IN> or B<SOCKADDR_IN6>
+structure, as returned by L<Socket/pack_sockaddr_in> or
+L<Socket/pack_sockaddr_in6>, which contains the address of the client that has
 connected and made the current request. This is provided "raw" in case you
 need it. While you could re-create it from C<peerhost>, it is readily
 available in both this server environment and the B<Apache::RPC::Server>
-environment and thus included for convenience.
+environment and thus included for convenience. Apply L<Socket/inet_ntop> to
+C<peerfamily> and this value to obtain textual representation of the address.
 
 =item peerhost
 
@@ -2384,8 +2428,8 @@ implements these, their help files and other information.
 The B<RPC::XML::Server> class now supports compressed messages, both incoming
 and outgoing. If a client indicates that it can understand compressed content,
 the server will use the B<Compress::Zlib> (available from CPAN) module, if
-available, to compress any outgoing messages above a certain threshhold in
-size (the default threshhold is set to 4096 bytes). The following methods are
+available, to compress any outgoing messages above a certain threshold in
+size (the default threshold is set to 4096 bytes). The following methods are
 all related to the compression support within the server class:
 
 =over 4
@@ -2398,7 +2442,7 @@ time, and cannot be changed.
 
 =item compress_thresh([MIN_LIMIT])
 
-Return or set the compression threshhold value. Messages smaller than this
+Return or set the compression threshold value. Messages smaller than this
 size in bytes will not be compressed, even when compression is available, to
 save on CPU resources. If a value is passed, it becomes the new limit and the
 old value is returned.
@@ -2410,7 +2454,7 @@ old value is returned.
 If the server anticipates handling large out-bound messages (for example, if
 the hosted code returns large Base64 values pre-encoded from file handles),
 the C<message_file_thresh> and C<message_temp_dir> settings may be used in a
-manner similar to B<RPC::XML::Client>. Specifically, the threshhold is used to
+manner similar to B<RPC::XML::Client>. Specifically, the threshold is used to
 determine when a message should be spooled to a filehandle rather than made
 into an in-memory string (the B<RPC::XML::base64> type can use a filehandle,
 thus eliminating the need for the data to ever be completely in memory). An
@@ -2419,7 +2463,7 @@ anonymous temporary file is used for these operations.
 Note that the message size is checked before compression is applied, since the
 size of the compressed output cannot be known until the full message is
 examined. It is possible that a message will be spooled even if its compressed
-size is below the threshhold, if the uncompressed size exceeds the threshhold.
+size is below the threshold, if the uncompressed size exceeds the threshold.
 
 =over 4
 
